##############################################################################
#         
# lpdedither - generate combined polarization images from dither sequence 
#             call ldedither for each polarization angle 
# 
# Author : Jose Acosta (jap@ll.iac.es)
# Version: 18. Feb. 2007
#         
##############################################################################
procedure lpdedither (input, output)
 
string  input	{prompt="Name of file list of dither images"}
string	output	{prompt="Name of output file"}
string  ftrimsec{"default",prompt="File containing sections of each polarization"} 
string	match	{"wcs",prompt="Method used to match images (wcs|manual|pick1|<filename>)"}
string  expmask	{"",prompt="Name of exposure mask (optional)"}
string	rejmask	{"",prompt="Namoe of rejection masks (optional)"}
string	subsky	{"combsky",enum="none|usrsky|combsky",prompt="Subtract sky"}
string	insky	{"",prompt="name of input sky list image"}

# Names for outputs saved images
string	outrow	 {"",prompt="prefix of image with pixel mapping corrected"}
string	outcrtk	 {"",prompt="prefix of row cross-talk corrected image "}
string	outprps	 {"",prompt="prefix of saved pre-post treated images"}
string  outfltc  {"f",prompt="prefix of flat-field corrected images"}
string	outmask	 {"m",prompt="prefix for individual object masks"}
string	outsbsk	 {"s",prompt="prefix of saved images - sky result"}
string	outcorr	 {"g",prompt="prefix of saved geometric distortion corrected images"}
string	outsky	 {"",prompt="name of output sky image"}
string	outshift {"",prompt="name of output image shift file"}

# Correction parameter files
string	inmask	 {"default",prompt="name of common mask file (default|<filename>)"}
string	incmask	 {"default",prompt="name of common fixpix mask file (default|<filename>)"}
string	inflat	 {"",prompt="name of flatfield correction file"}
string	intrans	 {"",prompt="name of geometric distortion correction file"}
bool 	corrow	 {no,prompt="Correct bad pixel mapping?"}
bool 	crtalk	 {no,prompt="Correct row cross talk?"}
bool	intprow	 {no,prompt="Interpolate central bad pixel rows"}
#bool	stmask	 {"yes",prompt="Make and apply individual star masks"}
string	skycomb  {"cycle",enum="cycle|run|single",prompt="Sky combination mode"}
int     nditpts  {5,prompt="Number of dithern points per cycle"}
bool	adjshift {yes,prompt="Adjust the calculated shift from image headers"}
#bool	surfsub	 {no,prompt="Surfit sky-substracted images"}
string  reject   {"none",enum="none|minmax|sigclip|pclip",prompt="Type of rejection when combining images"}
string  zero	 {"none",prompt="Image zero point offset when combining"}
string  scale	 {"none",prompt="Image scaling factor when combining"}

real	nlow     {0.05,prompt="Number or fraction of low pixels to reject to create image"}
real	nhigh    {0.25,prompt="Number of fraction of high pixels to reject to create image"}

pset    skycpars {prompt="Parameter set for sky image combination"}

real	xstr0	{1,prompt="Streching factor along x-axis for pol. vec. 0"}
real	ystr0	{1,prompt="Streching factor along y-axis for pol. vec. 0 "}
real	xstr90  {1,prompt="Streching factor along x-axis for pol. vec. 90"}
real	ystr90  {1,prompt="Streching factor along y-axis for pol. vec. 90"}
real	xstr135 {1,prompt="Streching factor along x-axis for pol. vec. 135"}
real	ystr135 {1,prompt="Streching factor along y-axis for pol. vec. 135"}
real	xstr45  {1,prompt="Streching factor along x-axis for pol. vec. 45"}
real	ystr45  {1,prompt="Streching factor along y-axis for pol. vec. 45"}

bool	verbose	{yes,prompt="Verbose"}

# Directories
string	tmpdir	{")_.tmpdir",prompt="Temporary directory for file conversion"}
string	stddir	{")_.stddir",prompt="Parametre directory"}


# check options
bool	dispstep{"no",prompt="Display dedither steps"}
pset	dispopt2{prompt="Option display"}
bool 	choffset{"no",prompt="Check for offset result?"}

# imcentroid parameters
int	bigbox	{17,prompt="Size of the coarse centering box"}
real    sherror	{5.,prompt="Maximum error in pixels on offset"}
real 	psf     {4.,prompt="Value of the PSF - FWHM (pixels) to find reference stars"}


string *list1	{prompt="Ignore this parameter(lpdedither.list1)"}
string *list2	{prompt="Ignore this parameter(lpdedither.list2)"}
string *list3	{prompt="Ignore this parameter(lpdedither.list3)"}


begin
string fsec,fsecext,trlist,comblist,flcolist,ldthlist
string inli,imorig,imosec,imext,combimg,shiftref
string lflat1,iflatsec,flattmp_0,flattmp_90,flattmp_45,flattmp_135
string intrans1,igdistsec,gdisttmp_0,gdisttmp_90,gdisttmp_45,gdisttmp_135
string imask1,ibp,bpmask1_0,bpmask1_90,bpmask1_45,bpmask1_135 
string bpmask1lst,bptrim1,match1,matchout
string oflc
int    isec,nditpts1
real   bdrx1,bdry1
bool   verb1

verb1 = verbose
nditpts1 = nditpts

if (ftrimsec == "default")
  fsec = "lirisdr$std/ipoltrim.dat"
else 
  fsec = ftrimsec
  
fsecext = "lirisdr$std/ipoltrim.ext"  
  
combimg = output  

if (outfltc != "")
 oflc = outfltc 
else  
 oflc = "f"
  
inli = mktemp(tmpdir//"lpddth")
sections(input,option="fullname",>inli)  

## correct pixel mapping


## create lists per section

comblist = mktemp(tmpdir//"_lpdolst")
flcolist = mktemp(tmpdir//"_lpdoflst")

ldthlist = mktemp(tmpdir//"_lpddthlst")
 
llistdiff("@"//inli,output=ldthlist)

list1 = inli    
list2 = ldthlist
while (fscan(list1,imorig) != EOF) 
  {
     if (fscan(list2,imext) == EOF)
        {
	   print "ERROR: Input files are not correct"
	   beep ; beep ; beep
	   bye
	}
	
     trlist = mktemp(tmpdir//"_lpddtrlst")
     # imext is the image name contains possible extensions,
     # imorig is the original name of image
     if (verb1) print("Trimming into  polarization vectors ",imext)
     ltrimpol(imext,imorig,fsec,outlist=trlist)
     ## adding wcs telescope coordinates
     lpwcsedit(imorig,instrument="liris")
     
     list3 = trlist
     while (fscan(list3,imosec) != EOF)
       {
	 imgets(imosec,"LPOLVECT")
	 isec = int(imgets.value)
         switch (isec) {
	 case 0: 
	    print(imosec,>>comblist//"_0")
	 case 90: 
	    print(imosec,>>comblist//"_90")
	 case 135: 
	    print(imosec,>>comblist//"_135")
	 case 45: 
	    print(imosec,>>comblist//"_45")
	 default :
	    {
	      print("Error: wrong number of image sections")
	      bye
	    }     
         }
       }
     delete(tmpdir//"_lpddtrlst*",verify-)  
  }

## Individual Flat fields will be applied to each piece 
if (inflat != "") 
  {
    lflat1 = mktemp(tmpdir//"_lpddthflt") 
    sections(inflat,option="fullname",>lflat1) 
    list1 = lflat1
    while(fscan(list1,iflatsec) != EOF)
      {
	 imgets(iflatsec,"LPOLVECT")
	 isec = int(imgets.value)
         switch (isec) {
	 case 0: 
	    flattmp_0 = iflatsec
	 case 90: 
	    flattmp_90 = iflatsec
	 case 135: 
	    flattmp_135 = iflatsec
	 case 45: 
	    flattmp_45 = iflatsec
	 default :
	    {
	      print("Error: wrong number of flat field images")
	      bye
	    }     
         }
      }
  }
else 
  {
    flattmp_0   = ""
    flattmp_90  = ""
    flattmp_135 = ""
    flattmp_45  = ""
  }  
  
print ("flattmp_90 ",flattmp_90)  
print ("flattmp_0 ",flattmp_0)  
print ("flattmp_45 ",flattmp_45)  
print ("flattmp_135 ",flattmp_135)  
  
## Bad pixels masks have to be cut in the same way as the images. 
## A complete image should be provided. 

if (inmask != "") 
  {
     imask1 = inmask
     bptrim1 = mktemp("_bptrim")
     bpmask1lst = mktemp("_bpmask1lst")
     print ("Trimming bad pixel image ",imask1)
     ltrimpol(imask1,bptrim1,fsec,outlist=bpmask1lst,ismask=yes)
     list1 = bpmask1lst
     while (fscan(list1,ibp) != EOF)
       {
         if (verb1) 
            print ("Creating bad pixel masks")
	 imgets(ibp,"LPOLVECT")
	 isec = int(imgets.value)
         switch (isec) {
	 case 0: 
	    bpmask1_0 = ibp
	 case 90: 
	    bpmask1_90 = ibp
	 case 135: 
	    bpmask1_135 = ibp
	 case 45: 
	    bpmask1_45 = ibp
	 default :
	    {
	      print("Error: wrong number of section images")
	      bye
	    }    
         }
       }
  }
else
  {
     bpmask1_0 = ""
     bpmask1_90 = ""
     bpmask1_135 = ""
     bpmask1_45 = ""
  }
  
## There will be a geometrical distortion correction to be applied to each 
## piece. A file containing the list of files must be provided.

if (intrans != "")
   {
      igdistsec=""
      isec = 999
      intrans1 = mktemp(tmpdir//"_lpddthtrn") 
      sections(intrans,option="fullname",>intrans1) 
      list1 = intrans1
    while(fscanf(list1,"%d %s",isec,igdistsec) != EOF)
      {
        print ("isec ",isec)
	print ("igdistsec ",igdistsec)
         switch (isec) {
	 case 0: 
	    gdisttmp_0 = igdistsec
	 case 90: 
	    gdisttmp_90 = igdistsec
	 case 135: 
	    gdisttmp_135 = igdistsec
	 case 45: 
	    gdisttmp_45 = igdistsec
	 default :
	    {
	      print("Error: wrong number of geometric distortion corrections")
	      bye
	    }     
         }
      }
  }
else 
  {
    gdisttmp_0   = ""
    gdisttmp_90  = ""
    gdisttmp_135 = ""
    gdisttmp_45  = ""
  }  

  
## now apply ldedither to each image section. 
   ## avoid to use vertical correction per quadrant just in case it is selected
   skycpars.gradquad=no 
## The offsets will be determined from the central sections (2 or 3) and
## applied to the rest. 
   #  90 deg. pol.
   shiftref = mktemp(tmpdir//"_shiftref")
   match1 = match
   matchout = match1
   bdrx1 = xstr90
   bdry1 = xstr90
   if (verb1) 
      print("Combining polarization at 90 deg.")
   ldedither("@"//comblist//"_90",combimg//"_90",match=match1,outsbsk=outsbsk,
         outshift=shiftref,outfltc=outfltc,outcorr=outcorr,
         subsky=subsky,inflat=flattmp_90,inmask=bpmask1_90,nditpts=nditpts1,
         intrans=gdisttmp_90,xstrech=bdrx1,ystrech=bdry1)
   #  0 deg. pol.
   if (match1 == "wcs")  matchout = shiftref
   bdrx1 = xstr0
   bdry1 = ystr0
   if (verb1) 
      print("Combining polarization at 0 deg.") 
   ldedither("@"//comblist//"_0",combimg//"_0",match=matchout,outsbsk=outsbsk,	 
         outshift="",outfltc=outfltc,outcorr=outcorr,
         subsky=subsky,inflat=flattmp_0,inmask=bpmask1_0,nditpts=nditpts1,
	 intrans=gdisttmp_0,xstrech=bdrx1,ystrech=bdry1) 
   #  135 deg. pol.
   bdrx1 = xstr135
   bdry1 = ystr135
   if (verb1) 
      print("Combining polarization at 135 deg.") 
   ldedither("@"//comblist//"_135",combimg//"_135",match=matchout,outsbsk=outsbsk,	 
         outshift="",outfltc=outfltc,outcorr=outcorr,
         subsky=subsky,inflat=flattmp_135,inmask=bpmask1_135,nditpts=nditpts1,
         intrans=gdisttmp_135,xstrech=bdrx1,ystrech=bdry1) 
   #  45 deg. pol.
   bdrx1 = xstr45
   bdry1 = ystr45
   if (verb1) 
      print("Combining polarization at 45 deg.") 
   ldedither("@"//comblist//"_45",combimg//"_45",match=matchout,outsbsk=outsbsk,	 
         outshift="",outfltc=outfltc,outcorr=outcorr,
         subsky=subsky,inflat=flattmp_45,inmask=bpmask1_45,nditpts=nditpts1,
         intrans=gdisttmp_45,xstrech=bdrx1,ystrech=bdry1) 



delete(tmpdir//"_lpdolst*",verify-)
delete(tmpdir//"_lpddth*",verify-)
delete(tmpdir//"_shiftref*",verify-)  

end
