# lmoswavecal - compute wavelength calibration files
# Author : Jose Acosta  (jap@ll.iac.es)
# Version: 01. May. 2007
#          
##############################################################################
procedure lmoswavecal(input,fmaskpos)
string	input		{prompt="List of comparison lamp spectra"}
string  fmaskpos	{prompt="File containing center of slitlets"}
string	coordlist	{"lirisdr$std/argon_ref.dat",prompt="Coordinate list"}
real	match		{30,prompt="Maximum difference for feature matching"}
string  database	{"database",prompt="Database in which to find feature data"}
string	specformat	{"lr_zj",enum="lr_zj|lr_hk|mr_k|hr_j|hr_h|hr_k",prompt="Grism used"}
bool	verbose		{yes,prompt="Verbose?"}

string *list1	{prompt="Ignore this parameter"}
string *list2	{prompt="Ignore this parameter"}

begin	
  string 	fmaskpos1, inli, arclet, slttyp, slttyp1, sltid1, coordlist1, database1
  string	specformat1
  real		xc, cdelt, crval, crval0, crquad, crpix0, xoff
  int		idumm	
  int		nsc
  bool		verb1
  
  
  # expand input image list
  inli = mktemp("_lmswvclinput")	
  sections(input,option="fullname", >inli)
  
  fmaskpos1 = fmaskpos
  if (!access(fmaskpos1)) 
    {
      print("ERROR: file ",fmaskpos1," does not exist")
      beep; beep; beep
      bye
    } 
 
    
  coordlist1 = coordlist
  specformat1 = specformat
  database1 = database

  verb1 = verbose

  ## start loop over slitlets
  
  list1 = inli
  list2 = fmaskpos1
  
  while (fscan(list1, arclet) != EOF) 
    {
        slttyp = "REF0"
	xc = 0 
        while (slttyp != "SLIT") 
	  {
	     nsc = fscan(list2, slttyp1, sltid1, xc, idumm, idumm, idumm, idumm)
	     slttyp = substr(slttyp1,1,4)
	  }
	
	xoff = 530. - xc
	if (specformat1 == "lr_zj") 
          {
	     cdelt = 6.1
	     crval0 = 12115.
	     crpix0 = 538 - xoff
	     crquad = 0.000
	  }
	if (specformat1 == "lr_hk")
	  {
	     cdelt = 9.71
	     crval0 = 18429.77.
	     crpix0 = 466.45 - xoff
	     crquad = 0.0000
	  }      
	if (specformat1 == "mr_k")
	  {
	     cdelt = 4.6
	     crval0 = 19300.
	     crpix0 = 530. - xoff
	     crquad = 0.0000
  	  }      
	if (specformat1 == "hr_j") 
	  {
	     cdelt = 1.81
	     crval0 = 12705.7604
	     crpix0 = 570.54 - xoff
	     crquad = 0.0000
  	  }      
	if (specformat1 == "hr_h")
	  {
	     cdelt = 2.57
	     crval0 = 16515.
	     crpix0 = 512. - xoff
	     crquad = 0.0000
  	  }      
	if (specformat1 == "hr_k")
	  {
	     cdelt = 3.55
	     crval0 = 22350.
	     crpix0 = 512 - xoff.
	     crquad = 0.0000
  	  }      
	crval = crval0 + cdelt * xoff  
        # call autoidentify based on the 
        aidpars.crpix = crpix0
	aidpars.crquad = crquad
	aidpars.reflist = "lirisdr$std/argon_ref.dat"
	aidpars.cddir = "sign"
	aidpars.aidord=3
        aidpars.ntarget = 20
	aidpars.nbins= 6
        if (verb1) {
 	  print ("  ")
	  print ("================================= ")
	  print ("arclet ",arclet)
	  print ("Doing wavelength calib. for slit id ",sltid1," using ",arclet)
	  print ("Slit center at col  ",xc)
	  print ("Ref. lambda is ",crval0," at Ref pixel ",crpix0)
	}
	autoidentify(arclet,crval0,cdelt,coordlist=coordlist1,section="middle line",
	    fwidth=5.,nsum=4,function="legendre",order=4,sample="*",niter=3,low_rej=3,
	    match=-20,high_rej=3.,dbwrite=yes,overwrite=yes,database=database1,threshold=500)

        # call reidentify
	reidentify(arclet,arclet,section="middle line",trace=yes,step=4,nsum=4,
	  nlost=4,coordlist=coordlist1,inter+) 
  
  
        # call fitcoords
	fitcoords(arclet,fitname="",interactive=yes,xorder=5,yorder=3)        

    } 
    
  
end  
  
  
  	
